class HueScene:
    def __init__(self, id, name, lights):
        self.id = id
        self.name = name
        self.lights = lights

    def __str__(self):
        num_lights = len(self.lights)
        result = f"Scene({self.id}) - {self.name} - {num_lights} lights"
        for light in self.lights:
            result = result + "\n" + light.name
        return result

    @staticmethod
    def group_scenes(scenes):
        scene_names = set([scene.name for scene in scenes])
        groups = {name: [] for name in scene_names}
        for key in groups:
            filtered_scenes = [scene for scene in scenes if scene.name == key]
            groups[key] += filtered_scenes
        return groups



class FailedToSetState(Exception):
    msg = "There was an error updating state"

class FailedToGetState(Exception):
    msg = "Failed to fetch state from api"

class UninitializedException(Exception):
    msg = "Hue API has not been initialized yet"

class ButtonNotPressedException(Exception):
    msg = "Link button was not pressed"

class DevicetypeException(Exception):
    msg = "Invalid devicetype"



import pickle
import os
import requests

import pytest

from hue_api import HueApi
from hue_api.lights import HueLight
from hue_api.exceptions import (UninitializedException,
                                DevicetypeException,
                                ButtonNotPressedException)


@pytest.fixture
def put_nothing(monkeypatch):
    """Requests.put() mocked to succeed and nothing else"""

    class MockResponse:
        status_code = 200

    def mock_put(*args, **kwargs):
        return MockResponse()
    monkeypatch.setattr(requests, "put", mock_put)

def test_load_existing():
    test_cache_file = '.test'
    test_ip_address = 'test_address'
    test_user_name = 'test_user_name'
    api = HueApi()
    with pytest.raises(UninitializedException):
        api.load_existing(cache_file=test_cache_file)

    with open(test_cache_file, 'wb') as pickle_file:
        cache = {
            'bridge_ip_address': test_ip_address,
            'user_name': test_user_name,
        }
        pickle.dump(cache, pickle_file)
    api.load_existing(cache_file=test_cache_file)
    assert api.user_name == test_user_name
    assert api.bridge_ip_address == test_ip_address
    assert api.base_url == 'http://test_address/api/test_user_name'
    os.remove(test_cache_file)

def test_create_new(monkeypatch):
    test_user_name = 'test_user_name'
    test_address = 'test_address'

    class MockResponse:
        def __init__(self, data):
            self.data = data

        def json(self):
            return [self.data]

    def mock_post_success(*args, **kwargs):
        assert args[0] == 'http://test_address/api'
        assert kwargs['json'].get('devicetype')
        return MockResponse({
            'success': {'username': test_user_name}
        })

    def mock_post_device_type_error(*args, **kwargs):
        return MockResponse({
            'error': {'type': 1}
        })

    def mock_post_button_not_pressed(*args, **kwargs):
        return MockResponse({
            'error': {'type': 101}
        })

    api = HueApi()
    monkeypatch.setattr(requests, 'post', mock_post_success)
    api.create_new_user(test_address)
    assert api.bridge_ip_address == test_address
    assert api.user_name == test_user_name
    assert api.base_url == 'http://test_address/api/test_user_name'

    monkeypatch.setattr(requests, 'post', mock_post_device_type_error)
    with pytest.raises(DevicetypeException):
        api.create_new_user(test_address)

    monkeypatch.setattr(requests, 'post', mock_post_button_not_pressed)
    with pytest.raises(ButtonNotPressedException):
        api.create_new_user(test_address)

def test_save_api_key():
    test_cache_file = '.test'
    test_address = 'test_address'
    test_user_name = 'test_user_name'
    api = HueApi()
    api.bridge_ip_address = test_address
    api.user_name = test_user_name
    api.save_api_key(cache_file=test_cache_file)
    with open(test_cache_file, 'rb') as cache:
        loaded = pickle.load(cache)
    assert loaded == {
        'bridge_ip_address': test_address,
        'user_name': test_user_name
    }
    os.remove(test_cache_file)

def test_fetch_lights(monkeypatch):
    test_light_name = 'test light'

    class MockResponse:
        def json(self):
            return {
                '1': {
                    'name': test_light_name,
                    'state': {}
                }
            }

    test_url = 'http://test.com'

    def mock_get(*args, **kwargs):
        assert args[0] == test_url + "/lights"
        return MockResponse()

    monkeypatch.setattr(requests, 'get', mock_get)
    api = HueApi()
    api.base_url = test_url
    api.fetch_lights()
    assert len(api.lights) == 1
    assert api.lights[0].name == test_light_name
    assert api.lights[0].id == 1

def test_filter_lights():
    api = HueApi()
    api.lights = [
        HueLight(1, 'Light 1', {}, None),
        HueLight(2, 'light 2', {}, None)
    ]
    assert len(api.filter_lights(None)) == 2
    assert len(api.filter_lights([1])) == 1
    assert api.filter_lights([1])[0].id == 1
    assert api.filter_lights([2])[0].id == 2
    assert api.filter_lights([1, 2]) == api.lights

def test_turn_on_off(put_nothing):
    api = HueApi()
    api.lights = [
        HueLight(1, 'Light 1', {}, None),
        HueLight(2, 'light 2', {}, None)
    ]
    for light in api.lights:
        assert not light.state.is_on
    api.turn_on()
    for light in api.lights:
        assert light.state.is_on
    api.turn_off([1])
    assert not api.lights[0].state.is_on
    assert api.lights[1].state.is_on

def test_toggle_on(put_nothing):
    api = HueApi()
    api.lights = [
        HueLight(1, 'Light 1', {'on': False}, None),
    ]
    light = api.lights[0]
    assert not light.state.is_on
    api.toggle_on()
    assert light.state.is_on
    api.toggle_on()
    assert not light.state.is_on

def test_set_brightness(put_nothing):
    api = HueApi()
    api.lights = [
        HueLight(1, 'Light 1', {'bri': 1}, None),
    ]
    light = api.lights[0]
    assert light.state.brightness == 1
    api.set_brightness(100)
    assert light.state.brightness == 100
    api.set_brightness('max')
    assert light.state.brightness == 254

def test_set_color(put_nothing):
    api = HueApi()
    api.lights = [
        HueLight(1, 'Light 1', {'hue': 0, 'sat': 0}, None),
    ]
    light = api.lights[0]
    assert light.state.hue == 0
    assert light.state.saturation == 0
    api.set_color('red')
    assert light.state.hue == 0
    assert light.state.saturation == 255
    api.set_color('green')
    assert light.state.hue == 21845
    assert light.state.saturation == 255



import colorsys
import inspect
import json
import os
import pickle
import uuid

import requests as re
import webcolors

import hue_api
from hue_api.lights import HueLight
from hue_api.groups import HueGroup
from hue_api.scene import HueScene
from hue_api.exceptions import (UninitializedException,
                                ButtonNotPressedException,
                                DevicetypeException)


class HueApi:

    def __init__(self):
        self.lights = []
        self.groups = []
        self.scenes = []
        self.grouped_scenes = {}

    def load_existing(self, *args, **kwargs):
        try:
            cache_file = kwargs.get('cache_file') or self.find_cache_file()
            with open(cache_file, 'rb') as cached_file:
                loaded = pickle.load(cached_file)
            bridge_ip_address = loaded.get('bridge_ip_address')
            user_name = loaded.get('user_name')
        except FileNotFoundError:
            raise UninitializedException
        self.bridge_ip_address = bridge_ip_address
        self.user_name = user_name
        self.base_url = f'http://{bridge_ip_address}/api/{user_name}'

    def find_cache_file(self):
        module_entry_point = inspect.getfile(hue_api)
        module_base_dir = os.path.dirname(module_entry_point)
        cache_file = os.path.join(module_base_dir, '.pickle')
        if not os.path.isfile(cache_file):
            raise FileNotFoundError
        return cache_file

    def create_new_user(self, bridge_ip_address, *args, **kwargs):
        url = f'http://{bridge_ip_address}/api'
        payload = {'devicetype': 'hue_cli'}
        response = re.post(url, json=payload)
        response = response.json()[0]
        error = response.get('error')
        if error:
            if error['type'] == 1:
                raise DevicetypeException
            else:
                raise ButtonNotPressedException
        user_name = response.get('success').get('username')
        self.user_name = user_name
        self.bridge_ip_address = bridge_ip_address
        self.base_url = f'http://{bridge_ip_address}/api/{user_name}'

    def save_api_key(self, *args, **kwargs):
        cache_file = kwargs.get('cache_file') or self.find_cache_file()
        with open(cache_file, 'wb') as pickle_file:
            cache = {
                'bridge_ip_address': self.bridge_ip_address,
                'user_name': self.user_name
            }
            pickle.dump(cache, pickle_file)

    def fetch_lights(self, *args, **kwargs):
        url = self.base_url + "/lights"
        response = re.get(url).json()
        lights = []
        for id in response:
            state = response[id].get('state')
            name = response[id].get('name')
            hue_light = HueLight(int(id), name, state, url)
            lights.append(hue_light)
        self.lights = lights
        return lights

    def fetch_groups(self, *args, **kwargs):
        url = self.base_url + "/groups"
        response = re.get(url).json()
        groups = []
        for id in response:
            group_name = response[id].get('name')
            lights = [int(light) for light in response[id].get('lights')]
            group_lights = self.filter_lights(lights)
            groups.append(HueGroup(id, group_name, group_lights))
        self.groups = groups
        return groups

    def fetch_scenes(self, *args, **kwargs):
        url = self.base_url + "/scenes"
        response = re.get(url).json()
        scenes = []
        for id in response:
            scene_name = response[id].get('name')
            lights = [int(light) for light in response[id].get('lights')]
            scene_lights = self.filter_lights(lights)
            scenes.append(HueScene(id, scene_name, scene_lights))
        self.scenes = scenes
        self.grouped_scenes = HueScene.group_scenes(scenes)
        return scenes

    def print_debug_info(self, *args, **kwargs):
        print(f"Bridge IP address: {self.bridge_ip_address}")
        print(f"Bridge API key (username): {self.user_name}")
        print(f"API Base URL: {self.base_url}")

    def list_lights(self, *args, **kwargs):
        for light in self.lights:
            print(light)

    def list_groups(self, *args, **kwargs):
        for group in self.groups:
            print(group, "\n")

    def list_scenes(self, *args, **kwargs):
        for scene in self.scenes:
            print(scene, "\n")

    def list_scene_groups(self, *args, **kwargs):
        for group in self.grouped_scenes:
            print(f"\nScene group:{group}")
            for scene in self.grouped_scenes[group]:
                print(scene)

    def filter_lights(self, indices):
        if not indices:
            return self.lights
        return [light for light in self.lights if light.id in indices]

    # Lights State Control

    def turn_on(self, indices=[]):
        for light in self.filter_lights(indices):
            light.set_on()

    def turn_off(self, indices=[]):
        for light in self.filter_lights(indices):
            light.set_off()

    def toggle_on(self, indices=[]):
        for light in self.filter_lights(indices):
            light.toggle_on()

    def set_brightness(self, brightness, indices=[]):
        if isinstance(brightness, str):
            try:
                brightness = float(brightness)
            except ValueError:
                pass
            if brightness == 'max':
                brightness = 254
            elif brightness == 'min':
                brightness = 1
            elif brightness == 'med':
                brightness = 127
        if isinstance(brightness, float):
            if brightness <= 1.0:
                brightness *= 254
            brightness = int(brightness)
        for light in self.filter_lights(indices):
            light.set_brightness(brightness)

    def set_color(self, color, indices=[]):
        if isinstance(color, str):
            color = webcolors.name_to_rgb(color)
            r = color[0] / 255
            g = color[1] / 255
            b = color[2] / 255
        else:
            r, g, b = color
        h, s, v = colorsys.rgb_to_hsv(r, g, b)
        hue = int((2**16 - 1) * h)
        saturation = int((2**8 - 1) * s)
        for light in self.filter_lights(indices):
            light.set_color(hue, saturation)



# LightState is reactive and bound to a specific light
class LightState:
    def __init__(self, state, bind_to=None):
        self.reachable = state.get('reachable')
        self.light = bind_to
        self.__brightness = state.get('bri')
        self.__hue = state.get('hue')
        self.__saturation = state.get('sat')
        self.__is_on = state.get('on')

    @property
    def brightness(self):
        return self.__brightness

    @brightness.setter
    def brightness(self, bri):
        self.__brightness = bri
        if self.light:
            self.light.set_state({'bri': bri})

    @property
    def color(self):
        return self.__hue, self.__sat

    @color.setter
    def color(self, color):
        hue, sat = color
        self.__hue = hue
        self.__sat = sat
        if self.light:
            self.light.set_state({'hue': hue, 'sat': sat})

    @property
    def hue(self):
        return self.__hue

    @hue.setter
    def hue(self, hue):
        self.__hue = hue
        if self.light:
            self.light.set_state({'hue': hue})

    @property
    def saturation(self):
        return self.__saturation

    @saturation.setter
    def saturation(self, sat):
        self.__saturation = sat
        if self.light:
            self.light.set_state({'sat': sat})

    @property
    def is_on(self):
        return self.__is_on

    @is_on.setter
    def is_on(self, on):
        self.__is_on = on
        if self.light:
            self.light.set_state({'on': on})

    def to_payload(self):
        payload = {
            'bri': self.brightness,
            'sat': self.saturation,
            'hue': self.hue,
            'on': self.is_on
        }
        return payload



class HueGroup:
    def __init__(self, id, name, lights):
        self.id = id
        self.name = name
        self.lights = lights

    def __str__(self):
        num_lights = len(self.lights)
        result = f"Group({self.id}) - {self.name} - {num_lights} lights"
        for light in self.lights:
            result = result + "\n" + light.__str__()
        return result



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

tests_require = ['pytest', 'pytest-cov']

setuptools.setup(
    name="hue_py",
    version="0.2.1",
    author="Matt Boran",
    author_email="mattboran@gmail.com",
    description="Python client for Philips Hue bulbs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mattboran/hue_py",
    download_url="https://github.com/mattboran/hue_py/releases/download/0.2.1/hue_py-0.2.1-py3-none-any.whl",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=[
        'requests>=2.23.0',
        'webcolors>=1.11'
    ],
    setup_requires=['pytest-runner'],
    tests_require=tests_require,
    extras_require={
        'test': tests_require
    },
    python_requires='>3.6.0',
)



import colorsys
import requests as re

from hue_api.exceptions import FailedToGetState, FailedToSetState
from hue_api.state import LightState

class HueLight:

    # Public methods

    def __init__(self, id, name, state_dict, base_url):
        self.id = id
        self.name = name
        self.light_url = f"{base_url}/{id}/"
        self.state = LightState(state_dict, bind_to=self)

    def __str__(self):
        string = f"{self.id} - {self.name}"
        if not self.state.reachable:
            status_string = " (unreachable)"
        else:
            status_string = " (on)" if self.state.is_on else " (off)"
        return string + status_string

    def toggle_on(self):
        self.state.is_on = not self.state.is_on

    def set_on(self):
        self.state.is_on = True

    def set_off(self):
        self.state.is_on = False

    # Provide values r, g, b between 0 and 1
    def set_color(self, hue, saturation):
        self.state.color = hue, saturation

    # Brightness should be between 0 and 254, but also accept between 0 and 1
    def set_brightness(self, brightness):
        self.state.brightness = brightness

    # Private methods

    # This is the reactive binding that gets called when a state value changes
    def set_state(self, state):
        try:
            state_url = self.light_url + "state/"
            response = re.put(state_url, json=state)
            status_code = response.status_code
            if status_code >= 300:
                raise FailedToSetState
            self.state = LightState(state, bind_to=self)
        except FailedToSetState as e:
            print(e.msg)
